@@ -7,15 +7,13 @@
  ** Modified by Marcos E. Wurzius & Philippe Lhoste
  **/
 
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdarg.h>
-#include <assert.h>
-#include <ctype.h>
+#include <cstdlib>
+#include <cassert>
+#include <cstring>
 
 #include <string>
 #include <string_view>
+#include <map>
 
 #include "ILexer.h"
 #include "Scintilla.h"
@@ -28,13 +26,18 @@
 #include "StyleContext.h"
 #include "CharacterSet.h"
 #include "LexerModule.h"
+#include "OptionSet.h"
+#include "DefaultLexer.h"
 
+using namespace Scintilla;
 using namespace Lexilla;
 
+namespace {
+
 // Test for [=[ ... ]=] delimiters, returns 0 if it's only a [ or ],
 // return 1 for [[ or ]], returns >=2 for [=[ or ]=] and so on.
 // The maximum number of '=' characters allowed is 254.
-static int LongDelimCheck(StyleContext &sc) {
+int LongDelimCheck(StyleContext &sc) {
 	int sep = 1;
 	while (sc.GetRelative(sep) == '=' && sep < 0xFF)
 		sep++;
@@ -43,31 +46,165 @@ static int LongDelimCheck(StyleContext &sc) {
 	return 0;
 }
 
-static void ColouriseLuaDoc(
-	Sci_PositionU startPos,
-	Sci_Position length,
-	int initStyle,
-	WordList *keywordlists[],
-	Accessor &styler) {
+const char * const luaWordListDesc[] = {
+	"Keywords",
+	"Basic functions",
+	"String, (table) & math functions",
+	"(coroutines), I/O & system facilities",
+	"user1",
+	"user2",
+	"user3",
+	"user4",
+    nullptr
+};
 
-	const WordList &keywords = *keywordlists[0];
-	const WordList &keywords2 = *keywordlists[1];
-	const WordList &keywords3 = *keywordlists[2];
-	const WordList &keywords4 = *keywordlists[3];
-	const WordList &keywords5 = *keywordlists[4];
-	const WordList &keywords6 = *keywordlists[5];
-	const WordList &keywords7 = *keywordlists[6];
-	const WordList &keywords8 = *keywordlists[7];
+const LexicalClass lexicalClasses[] = {
+	// Lexer Lua SCLEX_LUA SCE_LUA_:
+	0, "SCE_LUA_DEFAULT", "default", "White space: Visible only in View Whitespace mode (or if it has a back colour)",
+	1, "SCE_LUA_COMMENT", "comment", "Block comment (Lua 5.0)",
+	2, "SCE_LUA_COMMENTLINE", "comment line", "Line comment",
+	3, "SCE_LUA_COMMENTDOC", "comment documentation", "Doc comment",
+	4, "SCE_LUA_NUMBER", "literal numeric", "Number",
+	5, "SCE_LUA_WORD", "keyword", "Keyword",
+	6, "SCE_LUA_STRING", "literal string", "(Double quoted) String",
+	7, "SCE_LUA_CHARACTER", "literal string character", "Character (Single quoted string)",
+	8, "SCE_LUA_LITERALSTRING", "literal string", "Literal string",
+	9, "SCE_LUA_PREPROCESSOR", "preprocessor", "Preprocessor (obsolete in Lua 4.0 and up)",
+	10, "SCE_LUA_OPERATOR", "operator", "Operators",
+	11, "SCE_LUA_IDENTIFIER", "identifier", "Identifier (everything else...)",
+	12, "SCE_LUA_STRINGEOL", "error literal string", "End of line where string is not closed",
+	13, "SCE_LUA_WORD2", "identifier", "Other keywords",
+	14, "SCE_LUA_WORD3", "identifier", "Other keywords",
+	15, "SCE_LUA_WORD4", "identifier", "Other keywords",
+	16, "SCE_LUA_WORD5", "identifier", "Other keywords",
+	17, "SCE_LUA_WORD6", "identifier", "Other keywords",
+	18, "SCE_LUA_WORD7", "identifier", "Other keywords",
+	19, "SCE_LUA_WORD8", "identifier", "Other keywords",
+	20, "SCE_LUA_LABEL", "label", "Labels",
+};
+
+// Options used for LexerLua
+struct OptionsLua {
+	bool foldCompact = true;
+};
+
+struct OptionSetLua : public OptionSet<OptionsLua> {
+	OptionSetLua() {
+		DefineProperty("fold.compact", &OptionsLua::foldCompact);
+
+		DefineWordListSets(luaWordListDesc);
+	}
+};
+
+class LexerLua : public DefaultLexer {
+	WordList keywords;
+	WordList keywords2;
+	WordList keywords3;
+	WordList keywords4;
+	WordList keywords5;
+	WordList keywords6;
+	WordList keywords7;
+	WordList keywords8;
+	OptionsLua options;
+	OptionSetLua osLua;
+public:
+	explicit LexerLua() :
+		DefaultLexer("lua", SCLEX_LUA, lexicalClasses, ELEMENTS(lexicalClasses)) {
+	}
+	~LexerLua() override = default;
+	void SCI_METHOD Release() noexcept override {
+		delete this;
+	}
+	int SCI_METHOD Version() const noexcept override {
+		return lvRelease5;
+	}
+	const char *SCI_METHOD PropertyNames() noexcept override {
+		return osLua.PropertyNames();
+	}
+	int SCI_METHOD PropertyType(const char *name) override {
+		return osLua.PropertyType(name);
+	}
+	const char *SCI_METHOD DescribeProperty(const char *name) override {
+		return osLua.DescribeProperty(name);
+	}
+	Sci_Position SCI_METHOD PropertySet(const char *key, const char *val) override;
+	const char *SCI_METHOD PropertyGet(const char *key) override {
+		return osLua.PropertyGet(key);
+	}
+	const char *SCI_METHOD DescribeWordListSets() noexcept override {
+		return osLua.DescribeWordListSets();
+	}
+	Sci_Position SCI_METHOD WordListSet(int n, const char *wl) override;
+	void SCI_METHOD Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) override;
+	void SCI_METHOD Fold(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) override;
+
+	static ILexer5 *LexerFactoryLua() {
+		return new LexerLua();
+	}
+};
+
+Sci_Position SCI_METHOD LexerLua::PropertySet(const char *key, const char *val) {
+	if (osLua.PropertySet(&options, key, val)) {
+		return 0;
+	}
+	return -1;
+}
+
+Sci_Position SCI_METHOD LexerLua::WordListSet(int n, const char *wl) {
+	WordList *wordListN = nullptr;
+	switch (n) {
+	case 0:
+		wordListN = &keywords;
+		break;
+	case 1:
+		wordListN = &keywords2;
+		break;
+	case 2:
+		wordListN = &keywords3;
+		break;
+	case 3:
+		wordListN = &keywords4;
+		break;
+	case 4:
+		wordListN = &keywords5;
+		break;
+	case 5:
+		wordListN = &keywords6;
+		break;
+	case 6:
+		wordListN = &keywords7;
+		break;
+	case 7:
+		wordListN = &keywords8;
+		break;
+	default:
+		break;
+	}
+	Sci_Position firstModification = -1;
+	if (wordListN) {
+		if (wordListN->Set(wl)) {
+			firstModification = 0;
+		}
+	}
+	return firstModification;
+}
+
+constexpr int maskSeparator = 0xFF;
+constexpr int maskStringWs = 0x100;
+constexpr int maskDocComment = 0x200;
+
+void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) {
+	LexAccessor styler(pAccess);
 
 	// Accepts accented characters
-	CharacterSet setWordStart(CharacterSet::setAlpha, "_", true);
-	CharacterSet setWord(CharacterSet::setAlphaNum, "_", true);
+	const CharacterSet setWordStart(CharacterSet::setAlpha, "_", true);
+	const CharacterSet setWord(CharacterSet::setAlphaNum, "_", true);
 	// Not exactly following number definition (several dots are seen as OK, etc.)
 	// but probably enough in most cases. [pP] is for hex floats.
-	CharacterSet setNumber(CharacterSet::setDigits, ".-+abcdefpABCDEFP");
-	CharacterSet setExponent(CharacterSet::setNone, "eEpP");
-	CharacterSet setLuaOperator(CharacterSet::setNone, "*/-+()={}~[];<>,.^%:#&|");
-	CharacterSet setEscapeSkip(CharacterSet::setNone, "\"'\\");
+	const CharacterSet setNumber(CharacterSet::setDigits, ".-+abcdefpABCDEFP");
+	const CharacterSet setExponent("eEpP");
+	const CharacterSet setLuaOperator("*/-+()={}~[];<>,.^%:#&|");
+	const CharacterSet setEscapeSkip("\"'\\");
 
 	Sci_Position currentLine = styler.GetLine(startPos);
 	// Initialize long string [[ ... ]] or block comment --[[ ... ]],
@@ -76,11 +213,13 @@ static void ColouriseLuaDoc(
 	// Continuation of a string (\z whitespace escaping) is controlled by stringWs.
 	int sepCount = 0;
 	int stringWs = 0;
-	if (initStyle == SCE_LUA_LITERALSTRING || initStyle == SCE_LUA_COMMENT ||
-		initStyle == SCE_LUA_STRING || initStyle == SCE_LUA_CHARACTER) {
+	int lastLineDocComment = 0;
+	if ((currentLine > 0) &&
+		AnyOf(initStyle, SCE_LUA_DEFAULT, SCE_LUA_LITERALSTRING, SCE_LUA_COMMENT, SCE_LUA_COMMENTDOC, SCE_LUA_STRING, SCE_LUA_CHARACTER)) {
 		const int lineState = styler.GetLineState(currentLine - 1);
-		sepCount = lineState & 0xFF;
-		stringWs = lineState & 0x100;
+		sepCount = lineState & maskSeparator;
+		stringWs = lineState & maskStringWs;
+		lastLineDocComment = lineState & maskDocComment;
 	}
 
 	// results of identifier/keyword matching
@@ -90,7 +229,7 @@ static void ColouriseLuaDoc(
 	bool foundGoto = false;
 
 	// Do not leak onto next line
-	if (initStyle == SCE_LUA_STRINGEOL || initStyle == SCE_LUA_COMMENTLINE || initStyle == SCE_LUA_PREPROCESSOR) {
+	if (AnyOf(initStyle, SCE_LUA_STRINGEOL, SCE_LUA_COMMENTLINE, SCE_LUA_COMMENTDOC, SCE_LUA_PREPROCESSOR)) {
 		initStyle = SCE_LUA_DEFAULT;
 	}
 
@@ -104,12 +243,14 @@ static void ColouriseLuaDoc(
 			// Update the line state, so it can be seen by next line
 			currentLine = styler.GetLine(sc.currentPos);
 			switch (sc.state) {
+			case SCE_LUA_DEFAULT:
 			case SCE_LUA_LITERALSTRING:
 			case SCE_LUA_COMMENT:
+			case SCE_LUA_COMMENTDOC:
 			case SCE_LUA_STRING:
 			case SCE_LUA_CHARACTER:
 				// Inside a literal string, block comment or string, we set the line state
-				styler.SetLineState(currentLine, stringWs | sepCount);
+				styler.SetLineState(currentLine, lastLineDocComment | stringWs | sepCount);
 				break;
 			default:
 				// Reset the line state
@@ -141,20 +282,19 @@ static void ColouriseLuaDoc(
 				Sci_Position ln = 0;
 				while (IsASpaceOrTab(sc.GetRelative(ln)))	// skip over spaces/tabs
 					ln++;
-				Sci_Position ws1 = ln;
+				const Sci_Position ws1 = ln;
 				if (setWordStart.Contains(sc.GetRelative(ln))) {
-					int c, i = 0;
-					char s[100];
+					int c = 0;
+					std::string s;
 					while (setWord.Contains(c = sc.GetRelative(ln))) {	// get potential label
-						if (i < 90)
-							s[i++] = static_cast<char>(c);
+						s.push_back(static_cast<char>(c));
 						ln++;
 					}
-					s[i] = '\0'; Sci_Position lbl = ln;
+					const Sci_Position lbl = ln;
 					if (!keywords.InList(s)) {
 						while (IsASpaceOrTab(sc.GetRelative(ln)))	// skip over spaces/tabs
 							ln++;
-						Sci_Position ws2 = ln - lbl;
+						const Sci_Position ws2 = ln - lbl;
 						if (sc.GetRelative(ln) == ':' && sc.GetRelative(ln + 1) == ':') {
 							// final :: found, complete valid label construct
 							sc.ChangeState(SCE_LUA_LABEL);
@@ -206,14 +346,14 @@ static void ColouriseLuaDoc(
 					sc.Forward();
 					while (setWord.Contains(sc.ch))
 						sc.Forward();
-					char s[100];
-					sc.GetCurrent(s, sizeof(s));
+					std::string s;
+					sc.GetCurrentString(s, StyleContext::Transform::none);
 					if (keywords.InList(s))		// labels cannot be keywords
 						sc.ChangeState(SCE_LUA_WORD);
 				}
 				sc.SetState(SCE_LUA_DEFAULT);
 			}
-		} else if (sc.state == SCE_LUA_COMMENTLINE || sc.state == SCE_LUA_PREPROCESSOR) {
+		} else if (AnyOf(sc.state, SCE_LUA_COMMENTLINE, SCE_LUA_COMMENTDOC, SCE_LUA_PREPROCESSOR)) {
 			if (sc.atLineEnd) {
 				sc.ForwardSetState(SCE_LUA_DEFAULT);
 			}
@@ -227,7 +367,7 @@ static void ColouriseLuaDoc(
 					sc.Forward();
 				} else if (sc.chNext == 'z') {
 					sc.Forward();
-					stringWs = 0x100;
+					stringWs = maskStringWs;
 				}
 			} else if (sc.ch == '\"') {
 				sc.ForwardSetState(SCE_LUA_DEFAULT);
@@ -245,7 +385,7 @@ static void ColouriseLuaDoc(
 					sc.Forward();
 				} else if (sc.chNext == 'z') {
 					sc.Forward();
-					stringWs = 0x100;
+					stringWs = maskStringWs;
 				}
 			} else if (sc.ch == '\'') {
 				sc.ForwardSetState(SCE_LUA_DEFAULT);
@@ -277,9 +417,9 @@ static void ColouriseLuaDoc(
 				idenWordPos = 0;
 				idenStyle = SCE_LUA_IDENTIFIER;
 				foundGoto = false;
-				int cNext;
+				int cNext = 0;
 				do {
-					int c;
+					int c = 0;
 					const Sci_Position idenPosOld = idenPos;
 					std::string identSeg;
 					identSeg += static_cast<char>(sc.GetRelative(idenPos++));
@@ -287,29 +427,28 @@ static void ColouriseLuaDoc(
 						identSeg += static_cast<char>(c);
 						idenPos++;
 					}
-					if (keywords.InList(identSeg.c_str()) && (idenPosOld > 0)) {
+					if (keywords.InList(identSeg) && (idenPosOld > 0)) {
 						idenPos = idenPosOld - 1;	// keywords cannot mix
 						ident.pop_back();
 						break;
 					}
 					ident += identSeg;
-					const char* s = ident.c_str();
 					int newStyle = SCE_LUA_IDENTIFIER;
-					if (keywords.InList(s)) {
+					if (keywords.InList(ident)) {
 						newStyle = SCE_LUA_WORD;
-					} else if (keywords2.InList(s)) {
+					} else if (keywords2.InList(ident)) {
 						newStyle = SCE_LUA_WORD2;
-					} else if (keywords3.InList(s)) {
+					} else if (keywords3.InList(ident)) {
 						newStyle = SCE_LUA_WORD3;
-					} else if (keywords4.InList(s)) {
+					} else if (keywords4.InList(ident)) {
 						newStyle = SCE_LUA_WORD4;
-					} else if (keywords5.InList(s)) {
+					} else if (keywords5.InList(ident)) {
 						newStyle = SCE_LUA_WORD5;
-					} else if (keywords6.InList(s)) {
+					} else if (keywords6.InList(ident)) {
 						newStyle = SCE_LUA_WORD6;
-					} else if (keywords7.InList(s)) {
+					} else if (keywords7.InList(ident)) {
 						newStyle = SCE_LUA_WORD7;
-					} else if (keywords8.InList(s)) {
+					} else if (keywords8.InList(ident)) {
 						newStyle = SCE_LUA_WORD8;
 					}
 					if (newStyle != SCE_LUA_IDENTIFIER) {
@@ -326,7 +465,7 @@ static void ColouriseLuaDoc(
 						cNext = 0;
 					}
 				} while (cNext);
-				if ((idenStyle == SCE_LUA_WORD) && (ident.compare("goto") == 0)) {
+                if ((idenStyle == SCE_LUA_WORD) && (ident == "goto")) {
 					foundGoto = true;
 				}
 				sc.SetState(SCE_LUA_IDENTIFIER);
@@ -345,7 +484,7 @@ static void ColouriseLuaDoc(
 					sc.Forward(sepCount);
 				}
 			} else if (sc.Match('-', '-')) {
-				sc.SetState(SCE_LUA_COMMENTLINE);
+				sc.SetState(lastLineDocComment ? SCE_LUA_COMMENTDOC : SCE_LUA_COMMENTLINE);
 				if (sc.Match("--[")) {
 					sc.Forward(2);
 					sepCount = LongDelimCheck(sc);
@@ -353,6 +492,9 @@ static void ColouriseLuaDoc(
 						sc.ChangeState(SCE_LUA_COMMENT);
 						sc.Forward(sepCount);
 					}
+				} else if (sc.Match("---")) {
+					sc.SetState(SCE_LUA_COMMENTDOC);
+					lastLineDocComment = maskDocComment;
 				} else {
 					sc.Forward();
 				}
@@ -361,46 +503,51 @@ static void ColouriseLuaDoc(
 			} else if (setLuaOperator.Contains(sc.ch)) {
 				sc.SetState(SCE_LUA_OPERATOR);
 			}
+			if (!AnyOf(sc.state, SCE_LUA_DEFAULT, SCE_LUA_COMMENTDOC)) {
+				lastLineDocComment = 0;
+			}
 		}
 	}
 
 	sc.Complete();
 }
 
-static void FoldLuaDoc(Sci_PositionU startPos, Sci_Position length, int initStyle, WordList *[],
-                       Accessor &styler) {
-	const Sci_PositionU lengthDoc = startPos + length;
+void LexerLua::Fold(Sci_PositionU startPos_, Sci_Position length, int initStyle, IDocument *pAccess) {
+	LexAccessor styler(pAccess);
+	const Sci_Position startPos = startPos_;
+	const Sci_Position lengthDoc = startPos + length;
 	int visibleChars = 0;
 	Sci_Position lineCurrent = styler.GetLine(startPos);
 	int levelPrev = styler.LevelAt(lineCurrent) & SC_FOLDLEVELNUMBERMASK;
 	int levelCurrent = levelPrev;
 	char chNext = styler[startPos];
-	const bool foldCompact = styler.GetPropertyInt("fold.compact", 1) != 0;
+	const bool foldCompact = options.foldCompact;
 	int style = initStyle;
-	int styleNext = styler.StyleAt(startPos);
+	int styleNext = styler.StyleIndexAt(startPos);
 
-	for (Sci_PositionU i = startPos; i < lengthDoc; i++) {
+	for (Sci_Position i = startPos; i < lengthDoc; i++) {
 		const char ch = chNext;
 		chNext = styler.SafeGetCharAt(i + 1);
 		const int stylePrev = style;
 		style = styleNext;
-		styleNext = styler.StyleAt(i + 1);
+		styleNext = styler.StyleIndexAt(i + 1);
 		const bool atEOL = (ch == '\r' && chNext != '\n') || (ch == '\n');
 		if (style == SCE_LUA_WORD) {
-			if (ch == 'i' || ch == 'd' || ch == 'f' || ch == 'e' || ch == 'r' || ch == 'u') {
-				char s[10] = "";
-				for (Sci_PositionU j = 0; j < 8; j++) {
+			// Fixed list of folding words: if, do, function, repeat, end, until
+			// Must fix up next line with initial characters if any new words added.
+			if ((style != stylePrev) && AnyOf(ch, 'i', 'd', 'f', 'e', 'r', 'u')) {
+				std::string s;
+				for (Sci_Position j = 0; j < 8; j++) {	// 8 is length of longest: function
 					if (!iswordchar(styler[i + j])) {
 						break;
 					}
-					s[j] = styler[i + j];
-					s[j + 1] = '\0';
+					s.push_back(styler[i + j]);
 				}
 
-				if ((strcmp(s, "if") == 0) || (strcmp(s, "do") == 0) || (strcmp(s, "function") == 0) || (strcmp(s, "repeat") == 0)) {
+				if (s == "if" || s == "do" || s == "function" || s == "repeat") {
 					levelCurrent++;
 				}
-				if ((strcmp(s, "end") == 0) || (strcmp(s, "elseif") == 0) || (strcmp(s, "until") == 0)) {
+				if (s == "end" || s == "until") {
 					levelCurrent--;
 				}
 			}
@@ -439,49 +586,10 @@ static void FoldLuaDoc(Sci_PositionU startPos, Sci_Position length, int initStyl
 	}
 	// Fill in the real level of the next line, keeping the current flags as they will be filled in later
 
-	int flagsNext = styler.LevelAt(lineCurrent) & ~SC_FOLDLEVELNUMBERMASK;
+	const int flagsNext = styler.LevelAt(lineCurrent) & ~SC_FOLDLEVELNUMBERMASK;
 	styler.SetLevel(lineCurrent, levelPrev | flagsNext);
 }
 
-static const char * const luaWordListDesc[] = {
-	"Keywords",
-	"Basic functions",
-	"String, (table) & math functions",
-	"(coroutines), I/O & system facilities",
-	"user1",
-	"user2",
-	"user3",
-	"user4",
-	0
-};
-
-namespace {
-
-LexicalClass lexicalClasses[] = {
-	// Lexer Lua SCLEX_LUA SCE_LUA_:
-	0, "SCE_LUA_DEFAULT", "default", "White space: Visible only in View Whitespace mode (or if it has a back colour)",
-	1, "SCE_LUA_COMMENT", "comment", "Block comment (Lua 5.0)",
-	2, "SCE_LUA_COMMENTLINE", "comment line", "Line comment",
-	3, "SCE_LUA_COMMENTDOC", "comment documentation", "Doc comment -- Not used in Lua (yet?)",
-	4, "SCE_LUA_NUMBER", "literal numeric", "Number",
-	5, "SCE_LUA_WORD", "keyword", "Keyword",
-	6, "SCE_LUA_STRING", "literal string", "(Double quoted) String",
-	7, "SCE_LUA_CHARACTER", "literal string character", "Character (Single quoted string)",
-	8, "SCE_LUA_LITERALSTRING", "literal string", "Literal string",
-	9, "SCE_LUA_PREPROCESSOR", "preprocessor", "Preprocessor (obsolete in Lua 4.0 and up)",
-	10, "SCE_LUA_OPERATOR", "operator", "Operators",
-	11, "SCE_LUA_IDENTIFIER", "identifier", "Identifier (everything else...)",
-	12, "SCE_LUA_STRINGEOL", "error literal string", "End of line where string is not closed",
-	13, "SCE_LUA_WORD2", "identifier", "Other keywords",
-	14, "SCE_LUA_WORD3", "identifier", "Other keywords",
-	15, "SCE_LUA_WORD4", "identifier", "Other keywords",
-	16, "SCE_LUA_WORD5", "identifier", "Other keywords",
-	17, "SCE_LUA_WORD6", "identifier", "Other keywords",
-	18, "SCE_LUA_WORD7", "identifier", "Other keywords",
-	19, "SCE_LUA_WORD8", "identifier", "Other keywords",
-	20, "SCE_LUA_LABEL", "label", "Labels",
-};
-
 }
 
-LexerModule lmLua(SCLEX_LUA, ColouriseLuaDoc, "lua", FoldLuaDoc, luaWordListDesc, lexicalClasses, ELEMENTS(lexicalClasses));
+LexerModule lmLua(SCLEX_LUA, LexerLua::LexerFactoryLua, "lua", luaWordListDesc);
