@@ -339,7 +339,7 @@ bool IsValidRegOrQAdjacent(int ch) noexcept {
 
 /*
  * IsValidRegOrQPrecede
- * - returns true if ch is a valid preceeding character to put directly before Q / q
+ * - returns true if ch is a valid preceding character to put directly before Q / q
  *   * ref: Q Language: https://docs.raku.org/language/quoting
  */
 bool IsValidRegOrQPrecede(int ch) noexcept {
@@ -469,7 +469,7 @@ bool IsRegexStartAtScPos(StyleContext &sc, int &type, CharacterSet &set) {
 
 /*
  * IsValidIdentPrecede
- * - returns if ch is a valid preceeding char to put directly before an identifier
+ * - returns if ch is a valid preceding char to put directly before an identifier
  */
 bool IsValidIdentPrecede(int ch) noexcept {
 	return !(IsAlphaNumeric(ch) || ch == '_' || ch == '@' || ch == '$' || ch == '%');
@@ -486,7 +486,7 @@ bool IsValidDelimiter(int ch) noexcept {
 
 /*
  * GetDelimiterCloseChar
- * - returns the corrisponding close char for a given delimiter (could be the same char)
+ * - returns the corresponding close char for a given delimiter (could be the same char)
  */
 int GetDelimiterCloseChar(int ch) noexcept {
 	int ch_end = GetBracketCloseChar(ch);
@@ -498,7 +498,7 @@ int GetDelimiterCloseChar(int ch) noexcept {
 
 /*
  * GetRepeatCharCount
- * - returns the occurence count of match
+ * - returns the occurrence count of match
  */
 Sci_Position GetRepeatCharCount(StyleContext &sc, int chMatch, Sci_Position length) {
 	Sci_Position cnt = 0;
@@ -612,8 +612,8 @@ Sci_Position LengthToNextChar(StyleContext &sc, const Sci_Position length) {
 
 /*
  * GetRelativeString
- * - gets a relitive string and sets it in &str
- *   - resets string before seting
+ * - gets a relative string and sets it in &str
+ *   - resets string before setting
  */
 void GetRelativeString(StyleContext &sc, Sci_Position offset, Sci_Position length,
 		std::string &str) {
@@ -754,7 +754,7 @@ bool LexerRaku::IsOperatorChar(const int ch) {
  *   FIXME: *still* may not contain all valid characters
  */
 bool LexerRaku::IsWordChar(const int ch, bool allowNumber) {
-	// Unicode numbers should not apear in word identifiers
+	// Unicode numbers should not appear in word identifiers
 	if (ch > 0x7F) {
 		const CharacterCategory cc = CategoriseCharacter(ch);
 		switch (cc) {
@@ -1033,20 +1033,20 @@ Sci_Position LexerRaku::LengthToNonWordChar(StyleContext &sc, Sci_Position lengt
  */
 void SCI_METHOD LexerRaku::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) {
 	LexAccessor styler(pAccess);
-	DelimPair dpEmbeded;			// delimiter pair: embeded comments
+	DelimPair dpEmbeded;			// delimiter pair: embedded comments
 	DelimPair dpString;				// delimiter pair: string
 	DelimPair dpRegQ;				// delimiter pair: Regex / Q Lang
 	std::string hereDelim;			// heredoc delimiter (if in heredoc)
 	int hereState = 0;				// heredoc state to use (Q / QQ)
 	int numState = 0;				// number state / type
-	short cntDecimal = 0;			// number decinal count
+	short cntDecimal = 0;			// number decimal count
 	std::string wordLast;			// last word seen
 	std::string identLast;			// last identifier seen
 	std::string adverbLast;			// last (single) adverb seen
 	WordList lastAdverbs;			// last adverbs seen
 	Sci_Position len;				// temp length value
 	char s[100];					// temp char string
-	int typeDetect;					// temp type detected (for regex and Q lang)
+	int typeDetect = -1;			// temp type detected (for regex and Q lang)
 	Sci_Position lengthToEnd;		// length until the end of range
 
 	// Backtrack to safe start position before complex quoted elements
@@ -1216,7 +1216,7 @@ void SCI_METHOD LexerRaku::Lex(Sci_PositionU startPos, Sci_Position length, int
 						sc.Forward();
 					} else {
 						sc.SetState(SCE_RAKU_DEFAULT);
-						break; // too many decinal places
+						break; // too many decimal places
 					}
 				}
 				switch (numState) {
@@ -1463,7 +1463,7 @@ void SCI_METHOD LexerRaku::Lex(Sci_PositionU startPos, Sci_Position length, int
 				sc.SetState(hereState);
 			}
 
-			// Reset words: on operator simi-colon OR '}' (end of statement)
+			// Reset words: on operator semi-colon OR '}' (end of statement)
 			if (sc.state == SCE_RAKU_OPERATOR && (sc.ch == ';' || sc.ch == '}')) {
 				wordLast.clear();
 				identLast.clear();
@@ -1480,11 +1480,11 @@ void SCI_METHOD LexerRaku::Lex(Sci_PositionU startPos, Sci_Position length, int
 				sc.Forward(); // Condition met for "embedded comment"
 				dpEmbeded.opener = sc.ch;
 
-				// Find the opposite (termination) closeing bracket (if any)
+				// Find the opposite (termination) closing bracket (if any)
 				dpEmbeded.closer[0] = GetBracketCloseChar(dpEmbeded.opener);
 				if (dpEmbeded.closer[0] > 0) { // Enter "embedded comment"
 
-					// Find multiple opening character occurence
+					// Find multiple opening character occurrence
 					dpEmbeded.count = GetRepeatCharCount(sc, dpEmbeded.opener, lengthToEnd);
 					sc.SetState(SCE_RAKU_COMMENTEMBED);
 					sc.Forward(dpEmbeded.count - 1); // incremented in the next loop
