@@ -286,9 +286,11 @@ struct OptionSetSQL : public OptionSet<OptionsSQL> {
 
 		DefineProperty("fold.compact", &OptionsSQL::foldCompact);
 
-		DefineProperty("fold.sql.only.begin", &OptionsSQL::foldOnlyBegin);
+		DefineProperty("fold.sql.only.begin", &OptionsSQL::foldOnlyBegin,
+		               "Set to 1 to only fold on 'begin' but not other keywords.");
 
-		DefineProperty("lexer.sql.backticks.identifier", &OptionsSQL::sqlBackticksIdentifier);
+		DefineProperty("lexer.sql.backticks.identifier", &OptionsSQL::sqlBackticksIdentifier,
+		               "Recognise backtick quoting of identifiers.");
 
 		DefineProperty("lexer.sql.numbersign.comment", &OptionsSQL::sqlNumbersignComment,
 		               "If \"lexer.sql.numbersign.comment\" property is set to 0 a line beginning with '#' will not be a comment.");
