@@ -1332,7 +1332,7 @@ void Platform::DebugPrintf(const char *format, ...) noexcept
 	char buffer[2000];
 	va_list pArguments{};
 	va_start(pArguments, format);
-	vsprintf(buffer, format, pArguments);
+	vsnprintf(buffer, std::size(buffer), format, pArguments);
 	va_end(pArguments);
 	Platform::DebugDisplay(buffer);
 }
@@ -1345,7 +1345,7 @@ bool Platform::ShowAssertionPopUps(bool /*assertionPopUps*/) noexcept
 void Platform::Assert(const char *c, const char *file, int line) noexcept
 {
 	char buffer[2000];
-	sprintf(buffer, "Assertion [%s] failed at %s %d", c, file, line);
+	snprintf(buffer, std::size(buffer), "Assertion [%s] failed at %s %d", c, file, line);
 	if (Platform::ShowAssertionPopUps(false)) {
 		QMessageBox mb("Assertion Failure", buffer, QMessageBox::NoIcon,
 			QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
