@@ -19,9 +19,9 @@ typeAliases = {
 	"colour": "Colour",
 	"colouralpha": "ColourAlpha",
 	"findtext": "void *",
-	"findtextfull": "void *",
+	"findtextfull": "TextToFindFull *",
 	"formatrange": "void *",
-	"formatrangefull": "void *",
+	"formatrangefull": "RangeToFormatFull *",
 	"int": "int",
 	"keymod": "int",
 	"line": "Line",
@@ -30,7 +30,7 @@ typeAliases = {
 	"string": "const char *",
 	"stringresult": "char *",
 	"textrange": "void *",
-	"textrangefull": "void *",
+	"textrangefull": "TextRangeFull *",
 }
 
 basicTypes = [
@@ -55,7 +55,9 @@ deadValues = [
 ]
 
 def ActualTypeName(type, identifier=None):
-	if type in typeAliases:
+	if type == "pointer" and identifier in ["doc", "DocPointer", "CreateDocument"]:
+		return "IDocumentEditable *"
+	elif type in typeAliases:
 		return typeAliases[type]
 	else:
 		return type
@@ -63,6 +65,8 @@ def ActualTypeName(type, identifier=None):
 def IsEnumeration(s):
 	if s in ["Position", "Line", "Colour", "ColourAlpha"]:
 		return False
+	if s.endswith("*"):
+		return False
 	return s[:1].isupper()
 
 def JoinTypeAndIdentifier(type, identifier):
@@ -219,7 +223,7 @@ def HMethods(f):
 			if v["FeatureType"] in ["fun", "get", "set"]:
 				if v["FeatureType"] == "get" and name.startswith("Get"):
 					name = name[len("Get"):]
-				retType = ActualTypeName(v["ReturnType"])
+				retType = ActualTypeName(v["ReturnType"], name)
 				if IsEnumeration(retType):
 					retType = namespace + retType
 				parameters, args, callName = ParametersArgsCallname(v)
@@ -241,21 +245,21 @@ def CXXMethods(f):
 				msgName = "Message::" + name
 				if v["FeatureType"] == "get" and name.startswith("Get"):
 					name = name[len("Get"):]
-				retType = ActualTypeName(v["ReturnType"])
+				retType = ActualTypeName(v["ReturnType"], name)
 				parameters, args, callName = ParametersArgsCallname(v)
 				returnIfNeeded = "return " if retType != "void" else ""
 
 				out.append(JoinTypeAndIdentifier(retType, "ScintillaCall::" + name) + "(" + parameters + ")" + " {")
 				retCast = ""
 				retCastEnd = ""
-				if retType not in basicTypes or retType in ["int", "Colour", "ColourAlpha"]:
+				if retType.endswith("*"):
+					retCast = "reinterpret_cast<" + retType + ">("
+					retCastEnd = ")"
+				elif retType not in basicTypes or retType in ["int", "Colour", "ColourAlpha"]:
 					if IsEnumeration(retType):
 						retType = namespace + retType
 					retCast = "static_cast<" + retType + ">("
 					retCastEnd = ")"
-				elif retType in ["void *"]:
-					retCast = "reinterpret_cast<" + retType + ">("
-					retCastEnd = ")"
 				out.append("\t" + returnIfNeeded + retCast + callName + "(" + msgName + args + ")" + retCastEnd + ";")
 				out.append("}")
 				out.append("")
