@@ -8,12 +8,12 @@
 #ifndef LINEMARKER_H
 #define LINEMARKER_H
 
-namespace Scintilla {
+namespace Scintilla::Internal {
 
 class XPM;
 class RGBAImage;
 
-typedef void (*DrawLineMarkerFn)(Surface *surface, PRectangle &rcWhole, Font &fontForCharacter, int tFold, int marginStyle, const void *lineMarker);
+typedef void (*DrawLineMarkerFn)(Surface *surface, const PRectangle &rcWhole, const Font *fontForCharacter, int tFold, Scintilla::MarginType marginStyle, const void *lineMarker);
 
 /**
  */
@@ -21,11 +21,13 @@ class LineMarker {
 public:
 	enum class FoldPart { undefined, head, body, tail, headWithTail };
 
-	int markType = SC_MARK_CIRCLE;
-	ColourDesired fore = ColourDesired(0, 0, 0);
-	ColourDesired back = ColourDesired(0xff, 0xff, 0xff);
-	ColourDesired backSelected = ColourDesired(0xff, 0x00, 0x00);
-	int alpha = SC_ALPHA_NOALPHA;
+	Scintilla::MarkerSymbol markType = Scintilla::MarkerSymbol::Circle;
+	ColourRGBA fore = black;
+	ColourRGBA back = white;
+	ColourRGBA backSelected = ColourRGBA(0xff, 0x00, 0x00);
+	Scintilla::Layer layer = Scintilla::Layer::Base;
+	Scintilla::Alpha alpha = Scintilla::Alpha::NoAlpha;
+	XYPOSITION strokeWidth = 1.0f;
 	std::unique_ptr<XPM> pxpm;
 	std::unique_ptr<RGBAImage> image;
 	/** Some platforms, notably PLAT_CURSES, do not support Scintilla's native
@@ -41,10 +43,14 @@ public:
 	LineMarker &operator=(LineMarker&&) noexcept = default;
 	virtual ~LineMarker() = default;
 
+	ColourRGBA BackWithAlpha() const noexcept;
+
 	void SetXPM(const char *textForm);
 	void SetXPM(const char *const *linesForm);
 	void SetRGBAImage(Point sizeRGBAImage, float scale, const unsigned char *pixelsRGBAImage);
-	void Draw(Surface *surface, PRectangle &rcWhole, Font &fontForCharacter, FoldPart part, int marginStyle) const;
+	void AlignedPolygon(Surface *surface, const Point *pts, size_t npts) const;
+	void Draw(Surface *surface, const PRectangle &rcWhole, const Font *fontForCharacter, FoldPart part, Scintilla::MarginType marginStyle) const;
+	void DrawFoldingMark(Surface *surface, const PRectangle &rcWhole, FoldPart part) const;
 };
 
 }
