@@ -253,11 +253,9 @@ RESearch::RESearch(CharClassify *charClassTable) {
 	failure = 0;
 	charClass = charClassTable;
 	sta = NOP;                  /* status of lastpat */
-	bol = 0;
-	constexpr unsigned char nul = 0;
-	std::fill(bittab, std::end(bittab), nul);
-	std::fill(tagstk, std::end(tagstk), 0);
-	std::fill(nfa, std::end(nfa), '\0');
+	lineStartPos = 0;
+	lineEndPos = 0;
+	nfa[0] = END;
 	Clear();
 }
 
@@ -334,14 +332,11 @@ constexpr int isinset(const char *ap, unsigned char c) noexcept {
  * @return the char if it resolves to a simple char,
  * or -1 for a char class. In this case, bittab is changed.
  */
-int RESearch::GetBackslashExpression(
-    const char *pattern,
-    int &incr) noexcept {
+int RESearch::GetBackslashExpression(const char *pattern, int &incr) noexcept {
 	// Since error reporting is primitive and messages are not used anyway,
 	// I choose to interpret unexpected syntax in a logical way instead
 	// of reporting errors. Otherwise, we can stick on, eg., PCRE behaviour.
 	incr = 0;	// Most of the time, will skip the char "naturally".
-	int c = 0;
 	int result = -1;
 	const unsigned char bsc = *pattern;
 	if (!bsc) {
@@ -373,12 +368,12 @@ int RESearch::GetBackslashExpression(
 		}
 		break;
 	case 'd':
-		for (c = '0'; c <= '9'; c++) {
+		for (int c = '0'; c <= '9'; c++) {
 			ChSet(static_cast<unsigned char>(c));
 		}
 		break;
 	case 'D':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (c < '0' || c > '9') {
 				ChSet(static_cast<unsigned char>(c));
 			}
@@ -393,21 +388,21 @@ int RESearch::GetBackslashExpression(
 		ChSet('\v');
 		break;
 	case 'S':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (c != ' ' && !(c >= 0x09 && c <= 0x0D)) {
 				ChSet(static_cast<unsigned char>(c));
 			}
 		}
 		break;
 	case 'w':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (iswordc(static_cast<unsigned char>(c))) {
 				ChSet(static_cast<unsigned char>(c));
 			}
 		}
 		break;
 	case 'W':
-		for (c = 0; c < MAXCHR; c++) {
+		for (int c = 0; c < MAXCHR; c++) {
 			if (!iswordc(static_cast<unsigned char>(c))) {
 				ChSet(static_cast<unsigned char>(c));
 			}
@@ -419,34 +414,32 @@ int RESearch::GetBackslashExpression(
 	return result;
 }
 
-const char *RESearch::Compile(const char *pattern, Sci::Position length, bool caseSensitive, bool posix) noexcept {
-	char *mp=nfa;          /* nfa pointer       */
-	char *lp=nullptr;      /* saved pointer     */
-	char *sp=nfa;          /* another one       */
-	const char * mpMax = mp + MAXNFA - BITBLK - 10;
-
-	int tagi = 0;          /* tag stack index   */
-	int tagc = 1;          /* actual tag count  */
-
-	int n = 0;
-	char mask = 0;         /* xor mask -CCL/NCL */
-	int c1 = 0;
-	int c2 = 0;
-	int prevChar = 0;
-
+const char *RESearch::Compile(const char *pattern, Sci::Position length, bool caseSensitive, bool posix) {
 	if (!pattern || !length) {
 		if (sta)
 			return nullptr;
 		else
 			return badpat("No previous regular expression");
 	}
+
+	bittab.fill(0);
+	nfa[0] = END;
+
+	char *mp=nfa;          /* nfa pointer       */
+	char *sp=nfa;          /* another saved pointer */
+	const char * const mpMax = mp + MAXNFA - BITBLK - 10;
+
+	int tagstk[MAXTAG]{};  /* subpat tag stack */
+	int tagi = 0;          /* tag stack index   */
+	int tagc = 1;          /* actual tag count  */
+
 	sta = NOP;
 
 	const char *p=pattern;     /* pattern pointer   */
 	for (int i=0; i<length; i++, p++) {
 		if (mp > mpMax)
 			return badpat("Pattern too long");
-		lp = mp;
+		char *lp = mp;			/* saved pointer     */
 		switch (*p) {
 
 		case '.':               /* match any char  */
@@ -463,7 +456,7 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			break;
 
 		case '$':               /* match endofline */
-			if (!*(p+1)) {
+			if (!p[1]) {
 				*mp++ = EOL;
 			} else {
 				*mp++ = CHR;
@@ -471,17 +464,15 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			}
 			break;
 
-		case '[':               /* match char class */
-			*mp++ = CCL;
-			prevChar = 0;
+		case '[': {               /* match char class */
+			int prevChar = 0;
+			char mask = 0;      /* xor mask -CCL/NCL */
 
 			i++;
 			if (*++p == '^') {
 				mask = '\377';
 				i++;
 				p++;
-			} else {
-				mask = 0;
 			}
 
 			if (*p == '-') {	/* real dash */
@@ -500,13 +491,13 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 						// Previous def. was a char class like \d, take dash literally
 						prevChar = *p;
 						ChSet(*p);
-					} else if (*(p+1)) {
-						if (*(p+1) != ']') {
-							c1 = prevChar + 1;
+					} else if (p[1]) {
+						if (p[1] != ']') {
+							int c1 = prevChar + 1;
 							i++;
-							c2 = static_cast<unsigned char>(*++p);
+							int c2 = static_cast<unsigned char>(*++p);
 							if (c2 == '\\') {
-								if (!*(p+1)) {	// End of RE
+								if (!p[1]) {	// End of RE
 									return badpat("Missing ]");
 								} else {
 									i++;
@@ -543,7 +534,7 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 					} else {
 						return badpat("Missing ]");
 					}
-				} else if (*p == '\\' && *(p+1)) {
+				} else if (*p == '\\' && p[1]) {
 					i++;
 					p++;
 					int incr;
@@ -568,10 +559,12 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			if (!*p)
 				return badpat("Missing ]");
 
-			for (n = 0; n < BITBLK; bittab[n++] = 0)
-				*mp++ = static_cast<char>(mask ^ bittab[n]);
-
-			break;
+			*mp++ = CCL;
+			for (const unsigned char byte : bittab) {
+				*mp++ = mask ^ byte;
+			}
+			bittab.fill(0);
+		} break;
 
 		case '*':               /* match 0 or more... */
 		case '+':               /* match 1 or more... */
@@ -605,7 +598,7 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			while (--mp > lp)
 				*mp = mp[-1];
 			if (*p == '?')          *mp = CLQ;
-			else if (*(p+1) == '?') *mp = LCLO;
+			else if (p[1] == '?') *mp = LCLO;
 			else                    *mp = CLO;
 
 			mp = sp;
@@ -630,8 +623,8 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 			case '6':
 			case '7':
 			case '8':
-			case '9':
-				n = *p-'0';
+			case '9': {
+				const int n = *p-'0';
 				if (tagi > 0 && tagstk[tagi] == n)
 					return badpat("Cyclical reference");
 				if (tagc > n) {
@@ -640,7 +633,7 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 				} else {
 					return badpat("Undetermined reference");
 				}
-				break;
+			} break;
 			default:
 				if (!posix && *p == '(') {
 					if (tagc < MAXTAG) {
@@ -669,9 +662,10 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 						*mp++ = static_cast<unsigned char>(c);
 					} else {
 						*mp++ = CCL;
-						mask = 0;
-						for (n = 0; n < BITBLK; bittab[n++] = 0)
-							*mp++ = static_cast<char>(mask ^ bittab[n]);
+						for (const unsigned char byte : bittab) {
+							*mp++ = byte;
+						}
+						bittab.fill(0);
 					}
 				}
 			}
@@ -703,11 +697,12 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
 					*mp++ = CHR;
 					*mp++ = c;
 				} else {
-					*mp++ = CCL;
-					mask = 0;
 					ChSetWithCase(c, false);
-					for (n = 0; n < BITBLK; bittab[n++] = 0)
-						*mp++ = static_cast<char>(mask ^ bittab[n]);
+					*mp++ = CCL;
+					for (const unsigned char byte : bittab) {
+						*mp++ = byte;
+					}
+					bittab.fill(0);
 				}
 			}
 			break;
@@ -743,11 +738,9 @@ const char *RESearch::Compile(const char *pattern, Sci::Position length, bool ca
  *
  */
 int RESearch::Execute(const CharacterIndexer &ci, Sci::Position lp, Sci::Position endp) {
-	unsigned char c = 0;
 	Sci::Position ep = NOTFOUND;
 	const char * const ap = nfa;
 
-	bol = lp;
 	failure = 0;
 
 	Clear();
@@ -758,34 +751,54 @@ int RESearch::Execute(const CharacterIndexer &ci, Sci::Position lp, Sci::Positio
 		ep = PMatch(ci, lp, endp, ap);
 		break;
 	case EOL:			/* just searching for end of line normal path doesn't work */
-		if (ap[1] == END) {
+		if (endp == lineEndPos && ap[1] == END) {
 			lp = endp;
 			ep = lp;
 			break;
 		} else {
 			return 0;
 		}
-	case CHR:			/* ordinary char: locate it fast */
-		c = ap[1];
+	case CHR: {			/* ordinary char: locate it fast */
+		const unsigned char c = ap[1];
 		while ((lp < endp) && (static_cast<unsigned char>(ci.CharAt(lp)) != c))
 			lp++;
 		if (lp >= endp)	/* if EOS, fail, else fall through. */
 			return 0;
-		[[fallthrough]];
+	}
+	[[fallthrough]];
 	default:			/* regular matching all the way. */
 		while (lp < endp) {
 			ep = PMatch(ci, lp, endp, ap);
-			if (ep != NOTFOUND)
-				break;
+			if (ep != NOTFOUND) {
+				// fix match started from middle of character like DBCS trailing ASCII byte
+				const Sci::Position pos = ci.MovePositionOutsideChar(lp, -1);
+				if (pos != lp) {
+					ep = NOTFOUND;
+				} else {
+					break;
+				}
+			}
 			lp++;
 		}
 		break;
 	case END:			/* munged automaton. fail always */
 		return 0;
 	}
-	if (ep == NOTFOUND)
-		return 0;
+	if (ep == NOTFOUND) {
+		/* similar to EOL, match EOW at line end */
+		if (endp == lineEndPos && *ap == EOW) {
+			if ((ap[1] == END || ((ap[1] == EOL && ap[2] == END))) && iswordc(ci.CharAt(lp - 1))) {
+				lp = endp;
+				ep = lp;
+			} else {
+				return 0;
+			}
+		} else {
+			return 0;
+		}
+	}
 
+	ep = ci.MovePositionOutsideChar(ep, 1);
 	bopat[0] = lp;
 	eopat[0] = ep;
 	return 1;
@@ -831,14 +844,7 @@ int RESearch::Execute(const CharacterIndexer &ci, Sci::Position lp, Sci::Positio
 #define CCLSKIP 34	/* [CLO] CCL 32 bytes END */
 
 Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci::Position endp, const char *ap) {
-	int op = 0;
-	int c = 0;
-	int n = 0;
-	Sci::Position e = 0;		/* extra pointer for CLO  */
-	Sci::Position bp = 0;		/* beginning of subpat... */
-	Sci::Position ep = 0;		/* ending of subpat...    */
-	Sci::Position are = 0;	/* to save the line ptr.  */
-	Sci::Position llp = 0;	/* lazy lp for LCLO       */
+	unsigned char op = 0;
 
 	while ((op = *ap++) != END)
 		switch (op) {
@@ -859,39 +865,44 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 			ap += BITBLK;
 			break;
 		case BOL:
-			if (lp != bol)
+			if (lp != lineStartPos)
 				return NOTFOUND;
 			break;
 		case EOL:
-			if (lp < endp)
+			if (lp < lineEndPos)
 				return NOTFOUND;
 			break;
 		case BOT:
-			bopat[static_cast<int>(*ap++)] = lp;
+			if (lp != ci.MovePositionOutsideChar(lp, -1)) {
+				return NOTFOUND;
+			}
+			bopat[static_cast<unsigned char>(*ap++)] = lp;
 			break;
 		case EOT:
-			eopat[static_cast<int>(*ap++)] = lp;
+			lp = ci.MovePositionOutsideChar(lp, 1);
+			eopat[static_cast<unsigned char>(*ap++)] = lp;
 			break;
 		case BOW:
-			if ((lp!=bol && iswordc(ci.CharAt(lp-1))) || !iswordc(ci.CharAt(lp)))
+			if ((lp!=lineStartPos && iswordc(ci.CharAt(lp-1))) || !iswordc(ci.CharAt(lp)))
 				return NOTFOUND;
 			break;
 		case EOW:
-			if (lp==bol || !iswordc(ci.CharAt(lp-1)) || iswordc(ci.CharAt(lp)))
+			if (lp==lineStartPos || !iswordc(ci.CharAt(lp-1)) || iswordc(ci.CharAt(lp)))
 				return NOTFOUND;
 			break;
-		case REF:
-			n = *ap++;
-			bp = bopat[n];
-			ep = eopat[n];
+		case REF: {
+			const int n = static_cast<unsigned char>(*ap++);
+			Sci::Position bp = bopat[n];		/* beginning of subpat... */
+			const Sci::Position ep = eopat[n];	/* ending of subpat...    */
 			while (bp < ep)
 				if (ci.CharAt(bp++) != ci.CharAt(lp++))
 					return NOTFOUND;
-			break;
+		} break;
 		case LCLO:
 		case CLQ:
-		case CLO:
-			are = lp;
+		case CLO: {
+			int n = 0;
+			const Sci::Position are = lp;	/* to save the line ptr.  */
 			switch (*ap) {
 
 			case ANY:
@@ -903,15 +914,15 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 
 				n = ANYSKIP;
 				break;
-			case CHR:
-				c = ap[1];
+			case CHR: {
+				const char c = ap[1];
 				if (op == CLO || op == LCLO)
 					while ((lp < endp) && (c == ci.CharAt(lp)))
 						lp++;
 				else if ((lp < endp) && (c == ci.CharAt(lp)))
 					lp++;
 				n = CHRSKIP;
-				break;
+			} break;
 			case CCL:
 				while ((lp < endp) && isinset(ap+1, ci.CharAt(lp)))
 					lp++;
@@ -924,8 +935,8 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 			}
 			ap += n;
 
-			llp = lp;
-			e = NOTFOUND;
+			Sci::Position llp = lp;		/* lazy lp for LCLO       */
+			Sci::Position e = NOTFOUND; /* extra pointer for CLO  */
 			while (llp >= are) {
 				Sci::Position q;
 				if ((q = PMatch(ci, llp, endp, ap)) != NOTFOUND) {
@@ -939,11 +950,10 @@ Sci::Position RESearch::PMatch(const CharacterIndexer &ci, Sci::Position lp, Sci
 			if (*ap == EOT)
 				PMatch(ci, lp, endp, ap);
 			return e;
+		}
 		default:
 			//re_fail("RESearch::Execute: bad nfa.", static_cast<char>(op));
 			return NOTFOUND;
 		}
 	return lp;
 }
-
-
