@@ -81,11 +81,10 @@ struct FileWorker;
 // Scintilla documents can only be released by calling a method on a Scintilla
 // instance so store a Scintilla instance in the release functor
 struct BufferDocReleaser {
-	GUI::ScintillaWindow *pSci = nullptr;	// Non-owning
-	void operator()(void *pDoc) noexcept;
+	void operator()(SA::IDocumentEditable *pDoc) noexcept;
 };
 
-using BufferDoc = std::unique_ptr<void, BufferDocReleaser>;
+using BufferDoc = std::unique_ptr<SA::IDocumentEditable, BufferDocReleaser>;
 
 class Buffer {
 public:
@@ -344,7 +343,6 @@ protected:
 	StringList apis;
 	std::string apisFileNames;
 	std::string functionDefinition;
-	BufferDocReleaser docReleaser;
 
 	int diagnosticStyleStart;
 	enum { diagnosticStyles=4};
@@ -376,7 +374,7 @@ protected:
 	GUI::Window wStatusBar;
 	GUI::Window wTabBar;
 	GUI::Menu popup;
-	SA::Position contextPosition = SA::InvalidPosition;
+	int contextSelection = -1;
 	bool tbVisible;
 	bool tabVisible;
 	bool tabHideOne; // Hide tab bar if one buffer is opened only
@@ -499,8 +497,8 @@ protected:
 	BufferList buffers;
 
 	// Handle buffers
-	void *GetDocumentAt(BufferIndex index);
-	void SwitchDocumentAt(BufferIndex index, void *pdoc);
+	SA::IDocumentEditable *GetDocumentAt(BufferIndex index);
+	void SwitchDocumentAt(BufferIndex index, SA::IDocumentEditable *pdoc);
 	void SaveFolds(std::vector<SA::Line> &folds);
 	void RestoreFolds(const std::vector<SA::Line> &folds);
 	void UpdateBuffersCurrent();
@@ -659,7 +657,7 @@ protected:
 	SelectedRange GetSelectedRange();
 	void SetSelection(SA::Position anchor, SA::Position currentPos);
 	std::string GetCTag(GUI::ScintillaWindow *pw);
-	static void DropSelectionAt(GUI::ScintillaWindow &win, SA::Position position);
+	static void DropSelectionAt(GUI::ScintillaWindow &win, int selection);
 	virtual std::string GetRangeInUIEncoding(GUI::ScintillaWindow &win, SA::Span span);
 	static std::string GetLine(GUI::ScintillaWindow &win, SA::Line line);
 	void RangeExtend(GUI::ScintillaWindow &wCurrent, SA::Span &range,
