@@ -38,7 +38,6 @@
 using namespace Scintilla;
 using namespace Scintilla::Internal;
 
-#if !defined(_WIN32) && !defined(NO_CXX11_REGEX)
 // set global locale to pass std::regex related tests
 // see https://gcc.gnu.org/bugzilla/show_bug.cgi?id=63776
 struct GlobalLocaleInitializer {
@@ -48,7 +47,6 @@ struct GlobalLocaleInitializer {
 		} catch (...) {}
 	}
 } globalLocaleInitializer;
-#endif
 
 // Test Document.
 
@@ -92,6 +90,22 @@ std::string ReadFile(std::string path) {
 	return content;
 }
 
+struct Match {
+	Sci::Position location = 0;
+	Sci::Position length = 0;
+	constexpr Match() = default;
+	constexpr Match(Sci::Position location_, Sci::Position length_=0) : location(location_), length(length_) {
+	}
+	constexpr bool operator==(const Match &other) const {
+		return location == other.location && length == other.length;
+	}
+};
+
+std::ostream &operator << (std::ostream &os, Match const &value) {
+	os << value.location << "," << value.length;
+	return os;
+}
+
 struct DocPlus {
 	Document document;
 
@@ -130,6 +144,20 @@ struct DocPlus {
 		assert(*length == static_cast<Sci::Position>(needle.length()));
 		return document.FindText(document.Length(), 0, needle.data(), options, length);
 	}
+
+	Match FindString(Sci::Position minPos, Sci::Position maxPos, std::string_view needle, FindOption flags) {
+		Sci::Position lengthFinding = needle.length();
+		const Sci::Position location = document.FindText(minPos, maxPos, needle.data(), flags, &lengthFinding);
+		return { location, lengthFinding };
+	}
+
+	std::string Substitute(std::string_view substituteText) {
+		Sci::Position lengthsubstitute = substituteText.length();
+		std::string substituted = document.SubstituteByPosition(substituteText.data(), &lengthsubstitute);
+		assert(lengthsubstitute == static_cast<Sci::Position>(substituted.length()));
+		return substituted;
+	}
+
 	void MoveGap(Sci::Position gapNew) {
 		// Move gap to gapNew by inserting
 		document.InsertString(gapNew, "!", 1);
@@ -146,6 +174,8 @@ TEST_CASE("Document") {
 
 	constexpr std::string_view sText = "Scintilla";
 	constexpr Sci::Position sLength = sText.length();
+	constexpr FindOption rePosix = FindOption::RegExp | FindOption::Posix;
+	constexpr FindOption reCxx11 = FindOption::RegExp | FindOption::Cxx11RegEx;
 
 	SECTION("InsertOneLine") {
 		DocPlus doc("", 0);
@@ -468,50 +498,229 @@ TEST_CASE("Document") {
 
 	SECTION("RegexSearchAndSubstitution") {
 		DocPlus doc("\n\r\r\n 1a\xCE\x93z \n\r\r\n 2b\xCE\x93y \n\r\r\n", CpUtf8);// 1a gamma z 2b gamma y
-		constexpr std::string_view finding = R"(\d+(\w+))";
-		Sci::Position lengthFinding = finding.length();
-		Sci::Position location = doc.FindNeedle(finding, FindOption::RegExp | FindOption::Posix, &lengthFinding);
-		REQUIRE(location == 5);
-		REQUIRE(lengthFinding == 5);
+		const Sci::Position docLength = doc.document.Length();
+		Match match;
 
+		constexpr std::string_view finding = R"(\d+(\w+))";
 		constexpr std::string_view substituteText = R"(\t\1\n)";
-		Sci::Position lengthsubstitute = substituteText.length();
-		std::string substituted = doc.document.SubstituteByPosition(substituteText.data(), &lengthsubstitute);
-		REQUIRE(lengthsubstitute == 6);
+		constexpr std::string_view longest = "\\w+";
+		std::string substituted;
+
+		match = doc.FindString(0, docLength, finding, rePosix);
+		REQUIRE(match == Match(5, 5));
+		substituted = doc.Substitute(substituteText);
 		REQUIRE(substituted == "\ta\xCE\x93z\n");
 
-		lengthFinding = finding.length();
-		location = doc.FindNeedleReverse(finding, FindOption::RegExp | FindOption::Posix, &lengthFinding);
-		REQUIRE(location == 16);
-		REQUIRE(lengthFinding == 5);
-
-		lengthsubstitute = substituteText.length();
-		substituted = doc.document.SubstituteByPosition(substituteText.data(), &lengthsubstitute);
-		REQUIRE(lengthsubstitute == 6);
+		match = doc.FindString(docLength, 0, finding, rePosix);
+		REQUIRE(match == Match(16, 5));
+		substituted = doc.Substitute(substituteText);
 		REQUIRE(substituted == "\tb\xCE\x93y\n");
 
+		match = doc.FindString(docLength, 0, longest, rePosix);
+		REQUIRE(match == Match(16, 5));
+
 		#ifndef NO_CXX11_REGEX
-		lengthFinding = finding.length();
-		location = doc.FindNeedle(finding, FindOption::RegExp | FindOption::Cxx11RegEx, &lengthFinding);
-		REQUIRE(location == 5);
-		REQUIRE(lengthFinding == 5);
-
-		lengthsubstitute = substituteText.length();
-		substituted = doc.document.SubstituteByPosition(substituteText.data(), &lengthsubstitute);
-		REQUIRE(lengthsubstitute == 6);
+		match = doc.FindString(0, docLength, finding, reCxx11);
+		REQUIRE(match == Match(5, 5));
+		substituted = doc.Substitute(substituteText);
 		REQUIRE(substituted == "\ta\xCE\x93z\n");
 
-		lengthFinding = finding.length();
-		location = doc.FindNeedleReverse(finding, FindOption::RegExp | FindOption::Cxx11RegEx, &lengthFinding);
-		REQUIRE(location == 16);
-		REQUIRE(lengthFinding == 5);
-
-		lengthsubstitute = substituteText.length();
-		substituted = doc.document.SubstituteByPosition(substituteText.data(), &lengthsubstitute);
-		REQUIRE(lengthsubstitute == 6);
+		match = doc.FindString(docLength, 0, finding, reCxx11);
+		REQUIRE(match == Match(16, 5));
+		substituted = doc.Substitute(substituteText);
 		REQUIRE(substituted == "\tb\xCE\x93y\n");
+
+		match = doc.FindString(docLength, 0, longest, reCxx11);
+		REQUIRE(match == Match(16, 5));
 		#endif
 	}
+
+	SECTION("RegexAssertion") {
+		DocPlus doc("ab cd ef\r\ngh ij kl", CpUtf8);
+		const Sci::Position docLength = doc.document.Length();
+		Match match;
+
+		constexpr std::string_view findingBOL = "^";
+		match = doc.FindString(0, docLength, findingBOL, rePosix);
+		REQUIRE(match == Match(0));
+		match = doc.FindString(1, docLength, findingBOL, rePosix);
+		REQUIRE(match == Match(10));
+		match = doc.FindString(docLength, 0, findingBOL, rePosix);
+		REQUIRE(match == Match(10));
+		match = doc.FindString(docLength - 1, 0, findingBOL, rePosix);
+		REQUIRE(match == Match(10));
+
+		#ifndef NO_CXX11_REGEX
+		match = doc.FindString(0, docLength, findingBOL, reCxx11);
+		REQUIRE(match == Match(0));
+		match = doc.FindString(1, docLength, findingBOL, reCxx11);
+		REQUIRE(match == Match(10));
+		match = doc.FindString(docLength, 0, findingBOL, reCxx11);
+		REQUIRE(match == Match(10));
+		match = doc.FindString(docLength - 1, 0, findingBOL, reCxx11);
+		REQUIRE(match == Match(10));
+		#endif
+
+		constexpr std::string_view findingEOL = "$";
+		match = doc.FindString(0, docLength, findingEOL, rePosix);
+		REQUIRE(match == Match(8));
+		match = doc.FindString(1, docLength, findingEOL, rePosix);
+		REQUIRE(match == Match(8));
+		match = doc.FindString(docLength, 0, findingEOL, rePosix);
+		REQUIRE(match == Match(18));
+		match = doc.FindString(docLength - 1, 0, findingEOL, rePosix);
+		REQUIRE(match == Match(8));
+
+		#if !defined(NO_CXX11_REGEX) && !defined(_LIBCPP_VERSION)
+		match = doc.FindString(0, docLength, findingEOL, reCxx11);
+		REQUIRE(match == Match(8));
+		match = doc.FindString(1, docLength, findingEOL, reCxx11);
+		REQUIRE(match == Match(8));
+		match = doc.FindString(docLength, 0, findingEOL, reCxx11);
+		REQUIRE(match == Match(18));
+		match = doc.FindString(docLength - 1, 0, findingEOL, reCxx11);
+		REQUIRE(match == Match(8));
+		#endif
+
+		constexpr std::string_view findingBOW = "\\<";
+		match = doc.FindString(0, docLength, findingBOW, rePosix);
+		REQUIRE(match == Match(0));
+		match = doc.FindString(1, docLength, findingBOW, rePosix);
+		REQUIRE(match == Match(3));
+		match = doc.FindString(docLength, 0, findingBOW, rePosix);
+		REQUIRE(match == Match(16));
+		match = doc.FindString(docLength - 1, 0, findingBOW, rePosix);
+		REQUIRE(match == Match(16));
+
+		constexpr std::string_view findingEOW = "\\>";
+		match = doc.FindString(0, docLength, findingEOW, rePosix);
+		REQUIRE(match == Match(2));
+		match = doc.FindString(1, docLength, findingEOW, rePosix);
+		REQUIRE(match == Match(2));
+		match = doc.FindString(docLength, 0, findingEOW, rePosix);
+		REQUIRE(match == Match(18));
+		match = doc.FindString(docLength - 1, 0, findingEOW, rePosix);
+		REQUIRE(match == Match(15));
+
+		constexpr std::string_view findingEOWEOL = "\\>$";
+		match = doc.FindString(0, docLength, findingEOWEOL, rePosix);
+		REQUIRE(match == Match(8));
+		match = doc.FindString(10, docLength, findingEOWEOL, rePosix);
+		REQUIRE(match == Match(18));
+
+		#ifndef NO_CXX11_REGEX
+		constexpr std::string_view findingWB = "\\b";
+		match = doc.FindString(0, docLength, findingWB, reCxx11);
+		REQUIRE(match == Match(0));
+		match = doc.FindString(1, docLength, findingWB, reCxx11);
+		REQUIRE(match == Match(2));
+		match = doc.FindString(docLength, 0, findingWB, reCxx11);
+		#ifdef _LIBCPP_VERSION
+		REQUIRE(match == Match(16));
+		#else
+		REQUIRE(match == Match(18));
+		#endif
+		match = doc.FindString(docLength - 1, 0, findingWB, reCxx11);
+		REQUIRE(match == Match(16));
+
+		constexpr std::string_view findingNWB = "\\B";
+		match = doc.FindString(0, docLength, findingNWB, reCxx11);
+		REQUIRE(match == Match(1));
+		match = doc.FindString(1, docLength, findingNWB, reCxx11);
+		REQUIRE(match == Match(1));
+		#ifdef _LIBCPP_VERSION
+		match = doc.FindString(docLength, 0, findingNWB, reCxx11);
+		REQUIRE(match == Match(18));
+		match = doc.FindString(docLength - 1, 0, findingNWB, reCxx11);
+		REQUIRE(match == Match(14));
+		#else
+		match = doc.FindString(docLength, 0, findingNWB, reCxx11);
+		REQUIRE(match == Match(17));
+		match = doc.FindString(docLength - 1, 0, findingNWB, reCxx11);
+		REQUIRE(match == Match(17));
+		#endif
+		#endif
+	}
+
+	SECTION("RegexContextualAssertion") {
+		// For std::regex, check the use of assertions next to text in forward direction
+		// These are more common than empty assertions
+		DocPlus doc("ab cd ef\r\ngh ij kl", CpUtf8);
+		const Sci::Position docLength = doc.document.Length();
+		Match match;
+
+		#ifndef NO_CXX11_REGEX
+
+		match = doc.FindString(0, docLength, "^[a-z]", reCxx11);
+		REQUIRE(match == Match(0, 1));
+		match = doc.FindString(1, docLength, "^[a-z]", reCxx11);
+		REQUIRE(match == Match(10, 1));
+
+		match = doc.FindString(0, docLength, "[a-z]$", reCxx11);
+		REQUIRE(match == Match(7, 1));
+		match = doc.FindString(10, docLength, "[a-z]$", reCxx11);
+		REQUIRE(match == Match(17, 1));
+
+		match = doc.FindString(0, docLength, "\\b[a-z]", reCxx11);
+		REQUIRE(match == Match(0, 1));
+		match = doc.FindString(1, docLength, "\\b[a-z]", reCxx11);
+		REQUIRE(match == Match(3, 1));
+		match = doc.FindString(0, docLength, "[a-z]\\b", reCxx11);
+		REQUIRE(match == Match(1, 1));
+		match = doc.FindString(2, docLength, "[a-z]\\b", reCxx11);
+		REQUIRE(match == Match(4, 1));
+
+		match = doc.FindString(0, docLength, "\\B[a-z]", reCxx11);
+		REQUIRE(match == Match(1, 1));
+		match = doc.FindString(1, docLength, "\\B[a-z]", reCxx11);
+		REQUIRE(match == Match(1, 1));
+		match = doc.FindString(0, docLength, "[a-z]\\B", reCxx11);
+		REQUIRE(match == Match(0, 1));
+		match = doc.FindString(2, docLength, "[a-z]\\B", reCxx11);
+		REQUIRE(match == Match(3, 1));
+
+		#endif
+	}
+
+	SECTION("RESearchMovePositionOutsideCharUTF8") {
+		DocPlus doc(" a\xCE\x93\xCE\x93z ", CpUtf8);// a gamma gamma z
+		const Sci::Position docLength = doc.document.Length();
+		constexpr std::string_view finding = R"([a-z](\w)\1)";
+
+		Match match = doc.FindString(0, docLength, finding, rePosix);
+		REQUIRE(match == Match(1, 5));
+
+		constexpr std::string_view substituteText = R"(\t\1\n)";
+		std::string substituted = doc.Substitute(substituteText);
+		REQUIRE(substituted == "\t\xCE\x93\n");
+
+		#ifndef NO_CXX11_REGEX
+		match = doc.FindString(0, docLength, finding, reCxx11);
+		REQUIRE(match == Match(1, 5));
+
+		substituted = doc.Substitute(substituteText);
+		REQUIRE(substituted == "\t\xCE\x93\n");
+		#endif
+	}
+
+	SECTION("RESearchMovePositionOutsideCharDBCS") {
+		DocPlus doc(" \x98\x61xx 1aa\x83\xA1\x83\xA1z ", 932);// U+548C xx 1aa gamma gamma z
+		const Sci::Position docLength = doc.document.Length();
+
+		Match match = doc.FindString(0, docLength, R"([a-z](\w)\1)", rePosix);
+		REQUIRE(match == Match(8, 5));
+
+		constexpr std::string_view substituteText = R"(\t\1\n)";
+		std::string substituted = doc.Substitute(substituteText);
+		REQUIRE(substituted == "\t\x83\xA1\n");
+
+		match = doc.FindString(0, docLength, R"(\w([a-z])\1)", rePosix);
+		REQUIRE(match == Match(6, 3));
+
+		substituted = doc.Substitute(substituteText);
+		REQUIRE(substituted == "\ta\n");
+	}
+
 }
 
 TEST_CASE("Words") {
