@@ -41,6 +41,9 @@ public:
 	char CharAt(Sci::Position index) const override {
 		return s.at(index);
 	}
+	Sci::Position MovePositionOutsideChar(Sci::Position pos, [[maybe_unused]] Sci::Position moveDir) const noexcept override {
+		return pos;
+	}
 	std::string GetCharRange(Sci::Position position, Sci::Position lengthRetrieve) const {
 		return s.substr(position, lengthRetrieve);
 	}
@@ -60,6 +63,17 @@ TEST_CASE("RESearch") {
 		REQUIRE(nullptr == msg);
 	}
 
+	SECTION("Bug2413") {
+		// Check for https://sourceforge.net/p/scintilla/bugs/2413/
+		std::unique_ptr<RESearch> re = std::make_unique<RESearch>(&cc);
+		constexpr std::string_view BOW = "\\<";
+		constexpr std::string_view EOW = "\\>";
+		const char *msg = re->Compile(BOW.data(), BOW.length(), true, false);
+		REQUIRE(nullptr == msg);
+		msg = re->Compile(EOW.data(), EOW.length(), true, false);
+		REQUIRE(nullptr == msg);
+	}
+
 	SECTION("Execute") {
 		std::unique_ptr<RESearch> re = std::make_unique<RESearch>(&cc);
 		re->Compile(pattern.data(), pattern.length(), true, false);
