@@ -15,8 +15,8 @@ from scripts import Dependencies
 topComment = "# Created by AppDepGen.py. To recreate, run AppDepGen.py.\n"
 
 def Generate():
-	sciteSources = ["../src/*.cxx", "../lua/src/*.c"]
-	sciteIncludes = ["../../scintilla/include", "../src", "../lua/src"]
+	sciteSources = ["../src/*.cxx", "../lua/src/*.c", "../../lexilla/access/*.cxx", "../../scintilla/call/*.cxx"]
+	sciteIncludes = ["../../lexilla/include", "../../lexilla/access", "../../scintilla/include", "../src", "../lua/src"]
 
 	# Header magically injected into Lua builds on Win32 to make Unicode file names work
 	luaSubsts = { "LUA_USER_H": "scite_lua_win.h"}
