@@ -90,10 +90,25 @@ void SciTEWin::UpdateTabs(const std::vector<GUI::gui_string> &tabNames) {
 
 	// Avoiding drawing with WM_SETREDRAW here does not improve speed or flashing.
 
+	size_t tabDeleted = 0;
 	while (tabNames.size() < tabNamesCurrent.size()) {
 		// Remove extra tabs
 		TabCtrl_DeleteItem(HwndOf(wTabBar), tabChange);
 		tabNamesCurrent.erase(tabNamesCurrent.begin() + tabChange);
+		tabDeleted++;
+	}
+
+	// Dirty fix for bug #2347
+	if (tabDeleted > 0 && tabChange > 0 && tabChange == tabNames.size()) {
+		// Already deleted last tab, try to delete and insert the current last tab
+		TabCtrl_DeleteItem(HwndOf(wTabBar), tabChange - 1);
+
+		GUI::gui_string tabNameNext = tabNames.at(tabChange - 1);
+		TCITEMW tie {};
+		tie.mask = TCIF_TEXT | TCIF_IMAGE;
+		tie.iImage = -1;
+		tie.pszText = tabNameNext.data();
+		TabCtrl_InsertItem(HwndOf(wTabBar), tabChange - 1, &tie);
 	}
 
 	while (tabNames.size() > tabNamesCurrent.size()) {
